Get rid of an unused variable.
[obnox/wireshark/wip.git] / follow.c
index 256c30b6c9224ca2b2b366d1d98a8db17abfeb0d..c3b5ad0f3f42bb16b3e752aa8f35d1ba5b5283dd 100644 (file)
--- a/follow.c
+++ b/follow.c
@@ -1,13 +1,12 @@
 /* follow.c
  *
- * $Id: follow.c,v 1.19 1999/12/02 04:27:46 gerald Exp $
+ * $Id: follow.c,v 1.29 2002/02/28 19:35:08 gram Exp $
  *
  * Copyright 1998 Mike Hall <mlh@io.com>
  *
  * Ethereal - Network traffic analyzer
- * By Gerald Combs <gerald@zing.org>
+ * By Gerald Combs <gerald@ethereal.com>
  * Copyright 1998 Gerald Combs
- *
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +28,6 @@
 # include "config.h"
 #endif
 
-#include <gtk/gtk.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #endif
 
 #include <glib.h>
-#include "gtk/main.h"
-#include "packet.h"
+#include <epan/packet.h>
 #include "follow.h"
 
-extern FILE* data_out_file;
+FILE* data_out_file = NULL;
 
 gboolean incomplete_tcp_stream = FALSE;
 
-static guint32 ip_address[2];
+static guint8  ip_address[2][MAX_IPADDR_LEN];
 static u_int   tcp_port[2];
+static u_int   bytes_written[2];
+static gboolean is_ipv6 = FALSE;
 
 static int check_fragments( int, tcp_stream_chunk * );
-static void write_packet_data( tcp_stream_chunk *, const char * );
+static void write_packet_data( int, tcp_stream_chunk *, const char * );
+
+void
+follow_tcp_stats(follow_tcp_stats_t* stats)
+{
+       int i;
+
+       for (i = 0; i < 2 ; i++) {
+               memcpy(stats->ip_address[i], ip_address[i], MAX_IPADDR_LEN);
+               stats->tcp_port[i] = tcp_port[i];
+               stats->bytes_written[i] = bytes_written[i];
+               stats->is_ipv6 = is_ipv6;
+       }
+}
 
 /* this will build libpcap filter text that will only 
    pass the packets related to the stream. There is a 
@@ -62,7 +74,8 @@ static void write_packet_data( tcp_stream_chunk *, const char * );
    very good one */
 char* 
 build_follow_filter( packet_info *pi ) {
-  char* buf = malloc(1024);
+  char* buf = g_malloc(1024);
+  int len;
   if( pi->net_src.type == AT_IPv4 && pi->net_dst.type == AT_IPv4
        && pi->ipproto == 6 ) {
     /* TCP over IPv4 */
@@ -71,13 +84,26 @@ build_follow_filter( packet_info *pi ) {
             ip_to_str( pi->net_src.data), 
             ip_to_str( pi->net_dst.data), 
             pi->srcport, pi->destport );
+    len = 4;
+    is_ipv6 = FALSE;
+  }
+  else if( pi->net_src.type == AT_IPv6 && pi->net_dst.type == AT_IPv6
+       && pi->ipproto == 6 ) {
+    /* TCP over IPv6 */
+    sprintf( buf, 
+            "(ipv6.addr eq %s and ipv6.addr eq %s) and (tcp.port eq %d and tcp.port eq %d)",
+            ip6_to_str((struct e_in6_addr *)pi->net_src.data), 
+            ip6_to_str((struct e_in6_addr *)pi->net_dst.data), 
+            pi->srcport, pi->destport );
+    len = 16;
+    is_ipv6 = TRUE;
   }
   else { 
-    free( buf );
+    g_free( buf );
     return NULL;
   }
-  memcpy(&ip_address[0], pi->net_src.data, sizeof ip_address[0]);
-  memcpy(&ip_address[1], pi->net_dst.data, sizeof ip_address[1]);
+  memcpy(ip_address[0], pi->net_src.data, len);
+  memcpy(ip_address[1], pi->net_dst.data, len);
   tcp_port[0] = pi->srcport;
   tcp_port[1] = pi->destport;
   return buf;
@@ -87,44 +113,55 @@ build_follow_filter( packet_info *pi ) {
    session. We will try and handle duplicates, TCP fragments, and out 
    of order packets in a smart way. */
 
-static tcp_frag *frags[2] = { 0, 0};
+static tcp_frag *frags[2] = { 0, 0 };
 static u_long seq[2];
-static guint32 src[2] = { 0, 0 };
+static guint8 src_addr[2][MAX_IPADDR_LEN];
+static u_int src_port[2] = { 0, 0 };
 
 void 
 reassemble_tcp( u_long sequence, u_long length, const char* data,
                u_long data_length, int synflag, address *net_src,
-               address *net_dst, u_int srcport, u_int dstport,
-                guint32 secs, guint32 usecs) {
-  guint32 srcx, dstx;
-  int src_index, j, first = 0;
+               address *net_dst, u_int srcport, u_int dstport) {
+  guint8 srcx[MAX_IPADDR_LEN], dstx[MAX_IPADDR_LEN];
+  int src_index, j, first = 0, len;
   u_long newseq;
   tcp_frag *tmp_frag;
   tcp_stream_chunk sc;
   
   src_index = -1;
   
-  /* first check if this packet should be processed */
-  if (net_src->type != AT_IPv4 || net_dst->type != AT_IPv4)
+  /* First, check if this packet should be processed. */
+
+  if ((net_src->type != AT_IPv4 && net_src->type != AT_IPv6) ||
+      (net_dst->type != AT_IPv4 && net_dst->type != AT_IPv6))
     return;
-  memcpy(&srcx, net_src->data, sizeof srcx);
-  memcpy(&dstx, net_dst->data, sizeof dstx);
-  if ((srcx != ip_address[0] && srcx != ip_address[1]) ||
-      (dstx != ip_address[0] && dstx != ip_address[1]) ||
+
+  if (net_src->type == AT_IPv4)
+    len = 4;
+  else
+    len = 16;
+
+  /* Now check if the packet is for this connection. */
+  memcpy(srcx, net_src->data, len);
+  memcpy(dstx, net_dst->data, len);
+  if ((memcmp(srcx, ip_address[0], len) != 0 && 
+       memcmp(srcx, ip_address[1], len) != 0) ||
+      (memcmp(dstx, ip_address[0], len) != 0 &&
+       memcmp(dstx, ip_address[1], len) != 0) ||
       (srcport != tcp_port[0] && srcport != tcp_port[1]) ||
       (dstport != tcp_port[0] && dstport != tcp_port[1]))
     return;
 
   /* Initialize our stream chunk.  This data gets written to disk. */
-  sc.src_addr = srcx;
+  memcpy(sc.src_addr, srcx, len);
   sc.src_port = srcport;
-  sc.secs     = secs;
-  sc.usecs    = usecs;
   sc.dlen     = data_length;
 
-  /* first we check to see if we have seen this src ip before. */
+  /* Check to see if we have seen this source IP and port before.
+     (Yes, we have to check both source IP and port; the connection
+     might be between two different ports on the same machine.) */
   for( j=0; j<2; j++ ) {
-    if( src[j] == srcx ) {
+    if (memcmp(src_addr[j], srcx, len) == 0 && src_port[j] == srcport ) {
       src_index = j;
     }
   }
@@ -132,8 +169,9 @@ reassemble_tcp( u_long sequence, u_long length, const char* data,
   if( src_index < 0 ) {
     /* assign it to a src_index and get going */
     for( j=0; j<2; j++ ) {
-      if( src[j] == 0 ) {
-       src[j] = srcx;
+      if( src_port[j] == 0 ) {
+       memcpy(src_addr[j], srcx, len);
+       src_port[j] = srcport;
        src_index = j;
        first = 1;
        break;
@@ -158,7 +196,7 @@ reassemble_tcp( u_long sequence, u_long length, const char* data,
       seq[src_index]++;
     }
     /* write out the packet data */
-    write_packet_data( &sc, data );
+    write_packet_data( src_index, &sc, data );
     return;
   }
   /* if we are here, we have already seen this src, let's
@@ -184,6 +222,7 @@ reassemble_tcp( u_long sequence, u_long length, const char* data,
        data += new_len;
        data_length -= new_len;
       }
+      sc.dlen = data_length;
       sequence = seq[src_index];
       length = newseq - seq[src_index];
       
@@ -195,7 +234,7 @@ reassemble_tcp( u_long sequence, u_long length, const char* data,
     seq[src_index] += length;
     if( synflag ) seq[src_index]++;
     if( data ) {
-      write_packet_data( &sc, data );
+      write_packet_data( src_index, &sc, data );
     }
     /* done with the packet, see if it caused a fragment to fit */
     while( check_fragments( src_index, &sc ) )
@@ -203,7 +242,7 @@ reassemble_tcp( u_long sequence, u_long length, const char* data,
   }
   else {
     /* out of order packet */
-    if( sequence > seq[src_index] ) {
+    if(data_length > 0 && sequence > seq[src_index] ) {
       tmp_frag = (tcp_frag *)malloc( sizeof( tcp_frag ) );
       tmp_frag->data = (u_char *)malloc( data_length );
       tmp_frag->seq = sequence;
@@ -232,7 +271,7 @@ check_fragments( int index, tcp_stream_chunk *sc ) {
       /* this fragment fits the stream */
       if( current->data ) {
         sc->dlen = current->data_len;
-       write_packet_data( sc, current->data );
+       write_packet_data( index, sc, current->data );
       }
       seq[index] += current->len;
       if( prev ) {
@@ -258,9 +297,11 @@ reset_tcp_reassembly() {
   incomplete_tcp_stream = FALSE;
   for( i=0; i<2; i++ ) {
     seq[i] = 0;
-    src[i] = 0;
-    ip_address[i] = 0;
+    memset(src_addr[i], '\0', MAX_IPADDR_LEN);
+    src_port[i] = 0;
+    memset(ip_address[i], '\0', MAX_IPADDR_LEN);
     tcp_port[i] = 0;
+    bytes_written[i] = 0;
     current = frags[i];
     while( current ) {
       next = current->next;
@@ -273,7 +314,9 @@ reset_tcp_reassembly() {
 }
 
 static void 
-write_packet_data( tcp_stream_chunk *sc, const char *data ) {
+write_packet_data( int index, tcp_stream_chunk *sc, const char *data )
+{
   fwrite( sc, 1, sizeof(tcp_stream_chunk), data_out_file );
   fwrite( data, 1, sc->dlen, data_out_file );
+  bytes_written[index] += sc->dlen;
 }